/*********************************************************************************
 * Copyright (c) 2011 Forschungszentrum Juelich GmbH 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Forschungszentrum Juelich GmbH nor the names of its 
 * contributors may be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ********************************************************************************/
package eu.unicore.hila.grid.bes;

import java.io.File;

import de.fzj.unicore.uas.security.ClientProperties;
import eu.unicore.security.util.client.IClientProperties;

/**
 * @author bjoernh
 * 
 *         16.11.2011 10:47:48
 * 
 */
public class BesSecurityProperties extends ClientProperties implements
		IClientProperties {

	private static final String KEYSTORE_PROPERTY = BesProperties.BASE_BES_PROPERTY
			+ ".keystore";
	private static final String KEYSTORE_ALIAS_PROPERTY = BesProperties.BASE_BES_PROPERTY
			+ ".alias";
	private static final String KEYSTORE_PASSWORD_PROPERTY = BesProperties.BASE_BES_PROPERTY
			+ ".password";
	private BesProperties properties;

	/**
	 * 
	 */
	public BesSecurityProperties(BesProperties _props) {
		this.properties = _props;
	}

	/**
	 * @see de.fzj.unicore.uas.security.ClientProperties#getKeystore()
	 */
	@Override
	public String getKeystore() {
		String keyStore = properties.getProperty(KEYSTORE_PROPERTY,
				super.getKeystore());
		File ksFile = new File(keyStore);
		if (ksFile.isAbsolute()) {
			return keyStore;
		} else {
			return new File(((BesProperties) properties).getConfigBase(),
					keyStore).getAbsolutePath();
		}
	}

	/**
	 * @see de.fzj.unicore.uas.security.ClientProperties#getKeystoreAlias()
	 */
	@Override
	public String getKeystoreAlias() {
		return properties.getProperty(KEYSTORE_ALIAS_PROPERTY,
				super.getKeystoreAlias());
	}

	/**
	 * @see de.fzj.unicore.uas.security.ClientProperties#getKeystorePassword()
	 */
	@Override
	public String getKeystorePassword() {
		return properties.getProperty(KEYSTORE_PASSWORD_PROPERTY,
				super.getKeystorePassword());
	}
	
	/**
	 * @see de.fzj.unicore.uas.security.ClientProperties#getTruststore()
	 */
	@Override
	public String getTruststore() {
		return getKeystore();
	}
	
	/**
	 * @see de.fzj.unicore.uas.security.ClientProperties#getTruststorePassword()
	 */
	@Override
	public String getTruststorePassword() {
		return getKeystorePassword();
	}
	
	/**
	 * @see de.fzj.unicore.uas.security.ClientProperties#getTruststoreType()
	 */
	@Override
	public String getTruststoreType() {
		return getKeystoreType();
	}
	
	/**
	 * @see de.fzj.unicore.uas.security.ClientProperties#getKeystoreType()
	 */
	@Override
	public String getKeystoreType() {
		return super.getKeystoreType();
	}
}
