/*********************************************************************************
 * Copyright (c) 2010 Forschungszentrum Juelich GmbH 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Forschungszentrum Juelich GmbH nor the names of its 
 * contributors may be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ********************************************************************************/
package eu.unicore.hila.grid.unicore6;

import java.util.HashMap;
import java.util.Map;

import eu.unicore.hila.Location;
import eu.unicore.hila.Resource;
import eu.unicore.hila.annotations.ResourceType;

/**
 * @author bjoernh
 * 
 *         30.03.2010 13:33:12
 * 
 */
@ResourceType(locationStructure = {
	"unicore6:/sites/{site}/storages/{storage}/transfers/?",
	"unicore6:/{user}@sites/{site}/storages/{storage}/transfers/?",
	"unicore6:/sites/{site}/tasks/{task}/wd/transfers/?",
	"unicore6:/{user}@sites/{site}/tasks/{task}/wd/transfers/?",
	"unicore6:/storages/{storage}/transfers/?",
	"unicore6:/{user}@storages/{storage}/transfers/?" })
public class Unicore6TransfersCollection extends LocalTasksCollection implements
	Resource {

    private static final Map<Location, Unicore6TransfersCollection> transfersMap = new HashMap<Location, Unicore6TransfersCollection>();

    /**
     * 
     */
    private Unicore6TransfersCollection(Location _location) {
	super(_location);
    }

    public static synchronized Unicore6TransfersCollection locate(
	    Location _location, Object... extraInfo) {
	if (transfersMap.containsKey(_location)) {
	    return transfersMap.get(_location);
	} else {
	    final Unicore6TransfersCollection transferssCollection = new Unicore6TransfersCollection(
		    _location);
	    transfersMap.put(_location, transferssCollection);
	    return transferssCollection;
	}

    }

}
