/*********************************************************************************
 * Copyright (c) 2010-2011 Forschungszentrum Juelich GmbH 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Forschungszentrum Juelich GmbH nor the names of its 
 * contributors may be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ********************************************************************************/
package eu.unicore.hila.grid.unicore6;

import java.io.File;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

/**
 * @author bjoernh
 * 
 *         16.07.2010 14:10:54
 * 
 */
public class Unicore6SecurityPropertiesTest {

	/**
     * 
     */
	private static final String PASSWORD = "the!user";
	/**
     * 
     */
	private static final String KEYSTORE = "src/test/resources/user-keystore.jks";
	/**
     * 
     */
	private static final String ALIAS = "demo user";

	Unicore6SecurityProperties u6secprop = null;

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {
		Unicore6Properties u6props = new Unicore6Properties();
		u6props.setProperty(Unicore6SecurityProperties.HILA_UNICORE6_BASE
				+ Unicore6SecurityProperties.ALIAS, ALIAS);
		u6props.setProperty(Unicore6SecurityProperties.HILA_UNICORE6_BASE
				+ Unicore6SecurityProperties.KEYSTORE, KEYSTORE);
		u6props.setProperty(Unicore6SecurityProperties.HILA_UNICORE6_BASE
				+ Unicore6SecurityProperties.PASSWORD, PASSWORD);
		u6secprop = new Unicore6SecurityProperties(u6props);
		u6secprop.getETDSettings().setExtendTrustDelegation(true);
	}

	/**
	 * Test method for
	 * {@link eu.unicore.hila.grid.unicore6.Unicore6SecurityProperties#isSslEnabled()}
	 * .
	 */
	@Test
	public void testIsSslEnabled() {
		Assert.assertTrue(u6secprop.isSslEnabled());
	}

	/**
	 * Test method for
	 * {@link eu.unicore.hila.grid.unicore6.Unicore6SecurityProperties#getKeystore()}
	 * .
	 */
	@Test
	public void testGetKeystore() {
		Assert.assertNotNull(u6secprop.getKeystore());
		Assert.assertEquals(new File(KEYSTORE).getAbsolutePath(),
				u6secprop.getKeystore());
	}

	/**
	 * Test method for
	 * {@link eu.unicore.hila.grid.unicore6.Unicore6SecurityProperties#getKeystoreType()}
	 * .
	 */
	@Test
	public void testGetKeystoreType() {
		Assert.assertNotNull(u6secprop.getKeystoreType());
		Assert.assertEquals("JKS", u6secprop.getKeystoreType());
	}

	/**
	 * Test method for
	 * {@link eu.unicore.hila.grid.unicore6.Unicore6SecurityProperties#getKeystorePassword()}
	 * .
	 */
	@Test
	public void testGetKeystorePassword() {
		Assert.assertNotNull(u6secprop.getKeystorePassword());
		Assert.assertEquals(PASSWORD, u6secprop.getKeystorePassword());
	}

	/**
	 * Test method for
	 * {@link eu.unicore.hila.grid.unicore6.Unicore6SecurityProperties#getKeystoreAlias()}
	 * .
	 */
	@Test
	public void testGetKeystoreAlias() {
		Assert.assertNotNull(u6secprop.getKeystoreAlias());
		Assert.assertEquals(ALIAS, u6secprop.getKeystoreAlias());
	}

	/**
	 * Test method for
	 * {@link eu.unicore.hila.grid.unicore6.Unicore6SecurityProperties#getTruststore()}
	 * .
	 */
	@Test
	public void testGetTruststore() {
		Assert.assertNotNull(u6secprop.getTruststore());
		Assert.assertEquals(new File(KEYSTORE).getAbsolutePath(),
				u6secprop.getTruststore());
	}

	/**
	 * Test method for
	 * {@link eu.unicore.hila.grid.unicore6.Unicore6SecurityProperties#getTruststoreType()}
	 * .
	 */
	@Test
	public void testGetTruststoreType() {
		Assert.assertNotNull(u6secprop.getTruststoreType());
		Assert.assertEquals("JKS", u6secprop.getTruststoreType());
	}

	/**
	 * Test method for
	 * {@link eu.unicore.hila.grid.unicore6.Unicore6SecurityProperties#getTruststorePassword()}
	 * .
	 */
	@Test
	public void testGetTruststorePassword() {
		Assert.assertEquals(PASSWORD, u6secprop.getTruststorePassword());
	}

	/**
	 * Test method for
	 * {@link eu.unicore.hila.grid.unicore6.Unicore6SecurityProperties#isTrustDelegationEnabled()}
	 * .
	 */
	@Test
	public void testIsTrustDelegationEnabled() {
		Assert.assertTrue(u6secprop.getETDSettings().isExtendTrustDelegation());
	}

	/**
	 * Test method for
	 * {@link eu.unicore.hila.grid.unicore6.Unicore6SecurityProperties#getOutHandlerClassName()}
	 * .
	 */
	// @Test
	public void testGetOutHandlerClassName() {
		Assert.assertNotNull(u6secprop.getOutHandlerClassNames());
	}

}
